// @ myth - client - status.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/config.h"
#include "client/status.h"
#include "common/parse.h"
#include "common/range.h"
#include "common/error.h"
#include "common/format.h"

// @@ const

static const int StatusSize = 5;
static const int StatusPerCent = 15;

// @@ module VALUE

// @@@ class name_link

class name_link
{
    char* name;
    int length;
    name_link* next;
public:
    char* find(int width);
    name_link(PARSE* parse);
    ~name_link();
};

// @@@@ * name_link::find(int width)

char* name_link::find(int width)
{
    if(length <= width)
	return name;
    else if(next)
	return next->find(width);
    else
	return "*";
}

// @@@@ name_link::name_link(PARSE* parse)

name_link::name_link(PARSE* parse)
{
    parse->skip(INPUT_SPACE);
    char* s = parse->GetLine();
    length = strlen(s);
    name = strcpy(new char[length + 1], s);
    if(parse->good() && parse->LookAhead(INPUT_SPACE))
	next = new name_link(parse) ;
    else
	next = NULL;
}

// @@@@ name_link::~name_link()

name_link::~name_link()
{
    delete name;
    delete next;
}

// @@@ class range_link

class range_link : range
{
    name_link* name;
    range_link* next;
public:
    char* find(int width, status_value value);
    range_link(PARSE* parse);
    ~range_link();
};

// @@@@ * char* range_link::find(int width, status_value value)

char* range_link::find(int width, status_value value)
{
    if(member(value))
	return name->find(width);
    else if(next)
	return next->find(width, value);
    else
	return "ERR";
}

// @@@@ range_link::range_link(PARSE* parse)

range_link::range_link(PARSE* parse) : (parse->GetRange())
{
    name = new name_link(parse);

    if(parse->good() && parse->LookAhead(INPUT_RANGE))
	next = new range_link(parse);
    else
	next = NULL;
}

// @@@@ range_link::~range_link()

range_link::~range_link()
{
    delete name;
    delete next;
}

// @@@ class VALUE

class VALUE
{
    range_link* data[STATUS_END];
public:
    char* VALUE::find(int width, status_type type, status_value value);
    VALUE(STRING name);
};

// @@@@ * char* VALUE::find(int width, status_type type, status_value value)

char* VALUE::find(int width, status_type type, status_value value)
{
    if(data[type] == NULL)
	return "";
    else
	return data[type]->find(width, value);
}

// @@@@ VALUE::VALUE(STRING name)

VALUE::VALUE(STRING name)
{
    for(status_type type = 0; type < STATUS_END; type++)
	data[type] = NULL;

    PARSE parse(name);

    if(parse.LookAhead(INPUT_COMMENT))
	parse.skip(INPUT_COMMENT);

    while(parse.good())
    {
	status_type type = StatusToType(parse.GetWord());
	parse.skip(INPUT_COMMENT);
	data[type] = new range_link(&parse);
	if(parse.LookAhead(INPUT_COMMENT))
	    parse.skip(INPUT_COMMENT);
    }
    if(!parse.eof())
	bug.error("VALUE::VALUE", format("failed to parse '%s'", name()));
}

// @@@ data

// @@@@ static VALUE value

static VALUE value(format("%s/%s/%s",
			  HomeDirectory, LibraryDirectory, "STATUS"));

// @@ type

// @@@ * class STATUS

// @@@@ * void STATUS::set(status_type type, status_value value = TRUE)

void STATUS::set(status_type type, status_value value = TRUE)
{
    if(status[type] == value)
	return;

    if(touched == TRUE)
    {
	status[type] = value;
	return;
    }
    if(size().y >= STATUS_END)
	touched = TRUE;
    else
    {
	int tail = (from + size().y) % STATUS_END;
	
	if(tail > from)
	    touched = (type >= from && type < tail);
	else
	    touched = (type >= from || type < tail);
    }
    if(touched && strcmp(::value.find(size().x, type, value),
			 ::value.find(size().x, type, status[type])) == EQUAL)
	touched = FALSE;

    status[type] = value;
}

// @@@@ * void STATUS::show(int arg = 0)

void STATUS::show(int arg = 0)
{
    arg = abs(arg) % STATUS_END;

    if(arg != from)
    {
	from = arg;
	touched = TRUE;
    }
}

// @@@@ * void STATUS::update()

void STATUS::update()
{
    if(touched)
    {
	for(int y = 0; y < min(size().y, STATUS_END); y++)
	{
	    status_type type = (from + y) % STATUS_END;
	    print(coor(0, y), value.find(size().x, type, status[type]));
	}
	touched = FALSE;
    }
}

// @@@@ STATUS::STATUS()

STATUS::STATUS() : (WINDOW_LEFT, StatusSize, StatusPerCent)
{
    for(int i = 0; i < STATUS_END; i++)
	status[i] = FALSE;
    from = 0;
    touched = TRUE;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:

