// @ myth - common - symbol.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/symbol.h"
#include "common/error.h"
#include "common/format.h"

// @@ const

const char* const ALL_SYMBOLS = "!?=/([%*";
const char* const ALL_KINDS = "!?=/([%";

// @@ char NextLetter(char letter)

char NextLetter(char letter)
{
    switch(letter)
    {	
    case SYMBOL_BUNCH:
    case 'Z':
	return SYMBOL_BUNCH;
    case 'z':
	return 'A';
    default:
	return letter + 1;	// ASCII dependend, sorry.
    }
}

// @@ int SymbolToInt(symbol_type type)

int SymbolToInt(symbol_type type)
{
    switch(type)
    {
    case SYMBOL_POTION:
	return 0;
    case SYMBOL_SCROLL:
	return 1;
    case SYMBOL_RING:
	return 2;
    case SYMBOL_WAND:
	return 3;
    case SYMBOL_WEAPON:
	return 4;
    case SYMBOL_ARMOR:
	return 5;
    case SYMBOL_FOOD:
	return 6;
    case SYMBOL_BUNCH:
    case SYMBOL_EMPTY:
    case SYMBOL_ERROR:
    default:
	bug.error("::SymbolToInt", format("Unknown symbol %d", type));
	return ERROR;
    }
}

// @@ const char* SymbolName(symbol_type symbol, long count)

const char* SymbolName(symbol_type symbol, long count)
{
    if(count == SINGULAR)
    {
	switch(symbol)
	{
	case SYMBOL_POTION:
	    return "potion";
	case SYMBOL_SCROLL:
	    return "scroll";
	case SYMBOL_RING:
	    return "ring";
	case SYMBOL_WAND:
	    return "wand";
	case SYMBOL_WEAPON:
	    return "weapon";
	case SYMBOL_ARMOR:
	    return "armor";
	case SYMBOL_FOOD:
	    return "food";
	case SYMBOL_BUNCH:
	    return "bunch";
	case SYMBOL_EMPTY:
	    return "nothing";
	case SYMBOL_ERROR:
	default:
	    bug.error("::SymbolName", format("Unknown symbol %d", symbol));
	    return "bug";
	}
    }
    // count is PLURAL
    switch(symbol)
    {
    case SYMBOL_POTION:
	return "potions";
    case SYMBOL_SCROLL:
	return "scrolls";
    case SYMBOL_RING:
	return "rings";
    case SYMBOL_WAND:
	return "wands";
    case SYMBOL_WEAPON:
	return "weapons";
    case SYMBOL_ARMOR:
	return "armor";
    case SYMBOL_FOOD:
	return "food";
    case SYMBOL_BUNCH:
	return "bunches";
    case SYMBOL_EMPTY:
	return "nothing at all";
    case SYMBOL_ERROR:
    default:
	bug.error("::SymbolName", format("Unknown symbol %d", symbol));
    }
    return "bugs";
}	

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
