// @ myth - common - symbol.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef SYMBOL_HEADER
#define SYMBOL_HEADER

// @@ include

#include "common/common.h"

// @@ enum symbol_type

enum symbol_type
{
    SYMBOL_POTION	= '!',
    SYMBOL_SCROLL	= '?',
    SYMBOL_RING		= '=',
    SYMBOL_WAND		= '/',
    SYMBOL_WEAPON	= '(',
    SYMBOL_ARMOR	= '[',
    SYMBOL_FOOD		= '%',
    SYMBOL_BUNCH	= '*',
    SYMBOL_EMPTY	= ' ',
    SYMBOL_ERROR	= '\\'
};    

// @@ const

// A string of all symbols, including SYMBOL_BUNCH
extern const char* const ALL_SYMBOLS;
// A string of all symbols, excluding SYMBOL_BUNCH
extern const char* const ALL_KINDS;
// Number of symbols
static const int SymbolCount = 7;
// First usable letter
static const char FirstLetter = 'a';

// @@ char NextLetter(char letter)
//
// Return next char used for identification of objects in a bunch

extern char NextLetter(char letter);

// @@ int SymbolToInt(symbol_type type)
//
// Return a number < SymbolToInt uniqily identifying the symbol

extern int SymbolToInt(symbol_type type);

// @@ const char* SymbolName(symbol_type symbol, long count)
//
// Return the name of the objects represented by `symbol'

extern const char* SymbolName(symbol_type symbol, long count);

// @@ bool IsSymbol(char c)
//
// True if this is a valis symbol

inline bool IsSymbol(char c)
{ return strchr(ALL_SYMBOLS, c) != NULL; }

// @@ bool IsKind(char c)
//
// True, if this kind of symbol is used by objects

inline bool IsKind(char c)
{ return strchr(ALL_KINDS, c) != NULL; }

// @@ bool IsLetter(char c)
//
// True, iff `c' this is assigned to objects in list.

inline bool IsLetter(char c)
{ return isalpha(c); }

// @@ end

#endif SYMBOL_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
