// @ myth - server - corpse.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/corpse.h"
#include "common/binary.h"
#include "server/level.h"
#include "server/ray.h"
#include "common/format.h"

// @@ COCKATRICE

// @@@ void BODY_COCKATRICE::InsertAbility(MONSTER* monster)

void BODY_COCKATRICE::InsertAbility(MONSTER* monster)
{
    if(!monster->CheckAbility(ABILITY_STONE_PROTECTION))
    {
	monster->message("Touching a dead cockatrice is a bad move.");
	monster->AddAbility(new COUNTER_STONED(1L));
    }
}

// @@@ cont BODY_COCKATRICE::hurt(int power, int type, OBJECT* weapon,

cont BODY_COCKATRICE::hurt(int power, int type, OBJECT* weapon,
			   MONSTER* attack, MONSTER* defence, LEVEL* level)
{
    if(type & DAMAGE_ACID)
	power = power * 25 + 1;

    return BODY::hurt(power, type, weapon, attack, defence, level);
}

// @@ ACID_BLOB

// @@@ cont BODY_ACID_BLOB::hurt(int power, int type, OBJECT* weapon,

cont BODY_ACID_BLOB::hurt(int power, int type, OBJECT* weapon,
			  MONSTER* attack, MONSTER* defence, LEVEL* level)
{
    if(weapon != NULL && attack != NULL && level != NULL && rnd.chance(2, 3))
    {
	if(!NextTo(attack->position(), defence->position()))
	    weapon->damage();
	else if(weapon->IsBodyPart() || rnd.chance(1, 3))
	{
	    attack->message("You are splashed by the acid.");
	    STRING done = attack->hurt(power, &TheAcid, defence, level);
	    
	    if(done.length() > 0)
		level->WatchMonster(attack, done());
	}
	else
	{
	    attack->message(format("%s is covered in acid!",
				   ::print(weapon,
					   PRINT_THE | PRINT_UPPER |
					   PRINT_TYPE)));
	    weapon->damage();
	}
    }
    return BODY::hurt(power, type, weapon, attack, defence, level);
}

// @@ FUNGUS

// @@@ void BODY_FUNGUS::update(LEVEL* level, MONSTER* monster)

void BODY_FUNGUS::update(LEVEL* level, MONSTER* monster)
{
    BODY::update(level, monster);
    
    if(--TimeLeft < 1)
    {
	monster->message("You suddenly explode...");
	level->WatchMonster(monster, "%s suddenly explodes...");

	for(int i = rnd(1, rnd(1, 25)); i > 0; i--)
	{
	    level->zap(monster,
		       new RAY_FUNGUS(10 + monster->experience()),
		       monster->position() + coor(rnd(-25, 25), rnd(-25, 25)));
	}
	ReCharge();
    }
}	

// @@@ void BODY_FUNGUS::save(BINARY* binary)

void BODY_FUNGUS::save(BINARY* binary)
{
    BODY::save(binary);
    binary->PutShort(TimeLeft);
}

// @@@ BODY_FUNGUS::BODY_FUNGUS(BINARY* binary)

BODY_FUNGUS::BODY_FUNGUS(BINARY* binary) : (binary)
{
    TimeLeft = binary->GetShort();
}

// @@@ BODY_FUNGUS::BODY_FUNGUS()

BODY_FUNGUS::BODY_FUNGUS() : (1)
{
    AddAbility(new COUNTER_ACID_PROTECTION);
    AddAbility(new COUNTER_COLD_PROTECTION);
    AddAbility(new COUNTER_POISON_PROTECTION);
    ReCharge();
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
