// @ myth - server - invent.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef INVENT_HEADER
#define INVENT_HEADER

// @@ include

#include "common/command.h"
#include "server/specify.h"
#include "server/party.h"
#include "common/string.h"
#include "server/object.h"
#include "server/watch.h"

struct LEVEL;
struct PAGE;
struct QUESTION;
struct PARSE;

// @@ type

// pointer to void function(OBJECT*)
typedef void (*pfvpO)(OBJECT*);

// @@ INVENTORY

class INVENTORY
{
private:
    // List manipulating
    OBJECT_LIST list;
    void CleanUp();		// Add equal objects in the bunch
protected:
    void reset()
    { list.reset(); }
    OBJECT* current()
    { return list.current(); }
    void next()
    { list.next(); }
    int remain()
    { return list.remain(); }
    int length()
    { return list.length(); }
    void CreatePage(PAGE* page, PLAYER* player = NULL)
    { list.CreatePage(page, player); }
    virtual bool MayAdd(OBJECT*) // Don't add worn objects during CleanUp
    { return TRUE; }
private:
    // BitMap of letters used in the bunch
    long upper[SymbolCount];
    long lower[SymbolCount];
    // Manipulate the bitmaps
    void SetLetter(symbol_type symbol, char letter);
    bool CheckLetter(symbol_type symbol, char letter);
    void ClearLetter(symbol_type symbol, char letter);
    // Choose an unused letter, prefereable `letter'
    char ChooseLetter(symbol_type symbol, int letter = ERROR);
    // Initialise BitMap
    void create();
    // Keep track of when the pack changes
private:
    long NextEvent;		// Time for next event in this bunch
    bool IsLit;			// Contain light
    long MySize;		// Size of bunch
    long MyWeight;		// Weight of bunch
    char VisibleLook;		// How the largest visible item look
    char TotalLook;		// How the largers item look
protected:
    bool IsTouched;		// Changed sice last
public:
    void touch()
    { IsTouched = TRUE; }
    bool check()
    { return IsTouched || (NextEvent != 0L && NextEvent <= watch()); }
    virtual void update(LEVEL* level, coor pos, MONSTER* monster);
    // Maintain the bunch
protected:
    virtual void RemoveObject();
    virtual void AddObject(OBJECT* object);
public:
    bool empty()		// TRUE, iff no objects in inventory
    { return length() < 1; }
    bool light()		// TRUE, iff the bunch contain light
    { return IsLit; }
    long size()
    { return MySize; }
    long weight()
    { return MyWeight; }
    char look(bool SeeInvisible) // The char seen by the players
    { return SeeInvisible ? TotalLook : VisibleLook; }
    void send(inventory_type type, PLAYER* player);
    void add(OBJECT* object);
    void add(OBJECT_LIST* list);
    STRING add(OBJECT* object, print_flag flag, PLAYER* player = NULL);
    void add(OBJECT_LIST* list, PAGE* page, PLAYER* player = NULL);
    void remove(OBJECT* object);
    void remove(SPECIFY* specify, OBJECT_LIST* list,
		MONSTER* monster = NULL, long argument = 0L);
    bool contain(SPECIFY* specify);
    void free();
    void save(BINARY* binary);
    void load(BINARY* binary);
    void load(PARSE* parse, coor pos);
    INVENTORY();
    INVENTORY(PARSE* parse, coor pos);
    ~INVENTORY();
};    

// @@ end

#endif INVENT_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
