// @ myth - server - view.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef VIEW_HEADER
#define VIEW_HEADER

// @@ include

#include "common/ipc.h"
#include "common/map.h"
#include "common/picture.h"
#include "common/list.h"
#include "server/level.h"

// @@ mapping_type

enum mapping_type
{
    MAPPING_SIGHT,
    MAPPING_FORGET,
    MAPPING_OBJECT,
    MAPPING_ALL,
    MAPPING_ERROR = ERROR
};

// @@ (local) view_link

struct view_link
{
    coor where;
    char what;
    view_link(coor pos, char c)
    {
	where = pos;
	what = c;
    }
};

// @@ (local) VIEW_LIST
//
// Why do I have to make such local classes public to the world?
// Any clean way to avoid it?

class VIEW_LIST : LIST
{
    view_link* current()
    { return (view_link*) LIST::current(); }
public:
    void insert(coor pos, char c)
    { LIST::insert(new view_link(pos, c)); }
    void erase();
    void reset()
    { LIST::reset(); }
    inline void next()
    { LIST::next(); }
    bool check()
    { return LIST::current() != NULL; }
    coor where()
    { return VIEW_LIST::current()->where; }
    char what()
    { return VIEW_LIST::current()->what; }
    void clear();
    VIEW_LIST() { }
    ~VIEW_LIST()
    { clear(); }
};

// @@ VIEW

class VIEW
{
    // * Keep track of the levels
    MAP* map;		// A map of each level
    level_type depth;		// Current map to use
    coor home;			// Position on level
    VIEW_LIST list;		// List of temporary chars
    // * Control of sight
    PICTURE view;		// Specify what you can see
    int sight;			// Radius of looking distance
    int SightSquare;		// sight * sight
    coor SightCenter;		// Center of view
    coor SightCorner;		// Views position on the map
    // The view might have changed, update it
    void UpdateView(LEVEL* level);
    // Examine the view at pos
    void SetView(LEVEL* level, coor pos, bool light);
    // * Control of monster detection
    int detect;			// Monster detection radius
    int DetectSquare;		// detect * detect
    coor DetectCenter;		// Center of detection
    coor DetectCorner;		// Upper left corner of detection area
    // * Sending data to the player
    char buffer[MessageSize];	// Data to be send to the player
    int index;			// Index to the buffer
    coor PlotStart;		// Position of start of buffer
    // Plot char at position
    void send(coor where, char c, CHANNEL* channel);
    // Send buffer to channel
    void flush(CHANNEL* channel);
    // * Misc
    // Examine what a given position look like, to the player
    char examine(LEVEL* level, coor where, bool SeeInvisible);
    // Insert item in list of temporary thing
    void insert(coor where, char what, CHANNEL* channel);
public:
    // Player can see position.
    bool CanSee(coor where);
    // Player can detect monster at where
    bool CanDetect(coor where);
    // Update everything
    void update(LEVEL* level, PLAYER* player, CHANNEL* channel,
		bool SeeInvisible, mapping_type type, int length);
    // Move to a new level
    void enter(level_type type, CHANNEL* channel);
    // Forget all about this level
    void forget(CHANNEL* channel);
    // Move view to new position
    void move(LEVEL*, coor pos, CHANNEL* channel);
    // Map what is on a position, see it ot not!
    void plot(LEVEL* level, coor pos, bool SeeInvisible,
	      CHANNEL* channel = NULL);
    // Set maximum sight distance
    void look(int distance);
    // Set monster detection distance
    void detection(int distance);
    // * Administration
    void save(BINARY* binary);
    void load(BINARY* binary);
    VIEW();
    ~VIEW();
};

// @@ end

#endif VIEW_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
